"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.dotnet6 = void 0;
var code_builder_1 = require("../../../../helpers/code-builder");
var escape_1 = require("../../../../helpers/escape");
exports.dotnet6 = {
    info: {
        key: 'dotnet6',
        title: '.NET 6.0',
        link: 'https://docs.microsoft.com/en-us/dotnet/core/introduction',
        description: 'ReadMe Metrics Webhooks SDK usage on .NET 6.0'
    },
    convert: function (_a, options) {
        var secret = _a.secret, security = _a.security, server = _a.server;
        var opts = __assign({ indent: '  ' }, options);
        var _b = new code_builder_1.CodeBuilder({
            indent: opts.indent
        }), blank = _b.blank, endSection = _b.endSection, join = _b.join, push = _b.push, pushVariable = _b.pushVariable, ranges = _b.ranges, startSection = _b.startSection;
        push('var builder = WebApplication.CreateBuilder(args);');
        push('var app = builder.Build();');
        blank();
        push('// Your ReadMe secret');
        push("var secret = \"".concat(secret, "\";"));
        blank();
        push('app.MapPost("/webhook", async context =>');
        push('{');
        startSection('verification');
        push('// Verify the request is legitimate and came from ReadMe.', 1);
        push('var signature = context.Request.Headers["readme-signature"];', 1);
        push('var body = await new StreamReader(context.Request.Body).ReadToEndAsync();', 1);
        blank();
        push('try', 1);
        push('{', 1);
        push('ReadMe.Webhook.Verify(body, signature, secret);', 2);
        push('}', 1);
        push('catch (Exception e)', 1);
        push('{', 1);
        push('context.Response.StatusCode = StatusCodes.Status401Unauthorized;', 2);
        push('await context.Response.WriteAsJsonAsync(new', 2);
        push('{', 2);
        push('error = e.Message,', 3);
        push('});', 2);
        push('return;', 2);
        push('}', 1);
        endSection('verification');
        blank();
        startSection('payload');
        push('// Fetch the user from the database and return their data for use with OpenAPI variables.', 1);
        push('// @todo Write your own query logic to fetch a user by `body["email"]`.', 1);
        blank();
        push('await context.Response.WriteAsJsonAsync(new', 1);
        push('{', 1);
        if (!server.length && !security.length) {
            push('// Add custom data to return in your webhook call here.', 2);
        }
        if (server.length) {
            push('// OAS Server variables', 2);
            server.forEach(function (data) {
                pushVariable("".concat((0, escape_1.escapeForObjectKey)(data.name, true), " = \"").concat((0, escape_1.escapeForDoubleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "\","), {
                    type: 'server',
                    name: data.name,
                    indentationLevel: 2
                });
            });
        }
        if (server.length && security.length) {
            blank();
        }
        if (security.length) {
            push('// OAS Security variables', 2);
            security.forEach(function (data) {
                if (data.type === 'http') {
                    // Only HTTP Basic auth has any special handling for supplying auth.
                    if (data.scheme === 'basic') {
                        pushVariable("".concat((0, escape_1.escapeForObjectKey)(data.name, true), " = new { user = \"user\", pass = \"pass\" },"), {
                            type: 'security',
                            name: data.name,
                            indentationLevel: 2
                        });
                        return;
                    }
                }
                pushVariable("".concat((0, escape_1.escapeForObjectKey)(data.name, true), " = \"").concat((0, escape_1.escapeForDoubleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "\","), {
                    type: 'security',
                    name: data.name,
                    indentationLevel: 2
                });
            });
        }
        push('});', 1);
        endSection('payload');
        push('});');
        blank();
        push('app.Run($"http://localhost:8000");');
        blank();
        return {
            ranges: ranges(),
            snippet: join()
        };
    }
};
