"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.flask = void 0;
var code_builder_1 = require("../../../../helpers/code-builder");
var escape_1 = require("../../../../helpers/escape");
exports.flask = {
    info: {
        key: 'flask',
        title: 'Flask',
        link: 'https://flask.palletsprojects.com',
        description: 'ReadMe Metrics Webhooks SDK usage on Flask'
    },
    convert: function (_a, options) {
        var secret = _a.secret, security = _a.security, server = _a.server;
        var opts = __assign({ indent: '    ' }, options);
        var _b = new code_builder_1.CodeBuilder({
            indent: opts.indent
        }), blank = _b.blank, endSection = _b.endSection, join = _b.join, push = _b.push, pushVariable = _b.pushVariable, ranges = _b.ranges, startSection = _b.startSection;
        push('import os');
        push('import sys');
        push('from flask import Flask, request');
        push('from readme_metrics.VerifyWebhook import VerifyWebhook');
        blank();
        push('if os.getenv("README_API_KEY") is None:');
        push('sys.stderr.write("Missing `README_API_KEY` environment variable")', 1);
        push('sys.stderr.flush()', 1);
        push('sys.exit(1)', 1);
        blank();
        push('app = Flask(__name__)');
        blank();
        push('# Your ReadMe secret');
        push("secret = \"".concat(secret, "\""));
        blank();
        blank();
        push('@app.post("/webhook")');
        push('def webhook():');
        startSection('verification');
        push('# Verify the request is legitimate and came from ReadMe.', 1);
        push('signature = request.headers.get("readme-signature", None)', 1);
        blank();
        push('try:', 1);
        push('VerifyWebhook(request.get_json(), signature, secret)', 2);
        push('except Exception as error:', 1);
        push('return (', 2);
        push('{"error": str(error)},', 3);
        push('401,', 3);
        push('{"Content-Type": "application/json; charset=utf-8"},', 3);
        push(')', 2);
        endSection('verification');
        blank();
        startSection('payload');
        push('# Fetch the user from the database and return their data for use with OpenAPI variables.', 1);
        push('# user = User.objects.get(email__exact=request.values.get("email"))', 1);
        push('return (', 1);
        push('{', 2);
        if (!server.length && !security.length) {
            push('# Add custom data to return in your webhook call here.', 3);
        }
        if (server.length) {
            push('# OAS Server variables', 3);
            server.forEach(function (data) {
                pushVariable("\"".concat((0, escape_1.escapeForDoubleQuotes)(data.name), "\": \"").concat((0, escape_1.escapeForDoubleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "\","), {
                    type: 'server',
                    name: data.name,
                    indentationLevel: 3
                });
            });
        }
        if (server.length && security.length) {
            blank();
        }
        if (security.length) {
            push('# OAS Security variables', 3);
            security.forEach(function (data) {
                if (data.type === 'http') {
                    // Only HTTP Basic auth has any special handling for supplying auth.
                    if (data.scheme === 'basic') {
                        pushVariable("\"".concat((0, escape_1.escapeForDoubleQuotes)(data.name), "\": {\"user\": \"user\", \"pass\": \"pass\"},"), {
                            type: 'security',
                            name: data.name,
                            indentationLevel: 3
                        });
                        return;
                    }
                }
                pushVariable("\"".concat((0, escape_1.escapeForDoubleQuotes)(data.name), "\": \"").concat((0, escape_1.escapeForDoubleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "\","), {
                    type: 'security',
                    name: data.name,
                    indentationLevel: 3
                });
            });
        }
        push('},', 2);
        push('200,', 2);
        push('{"Content-Type": "application/json; charset=utf-8"},', 2);
        push(')', 1);
        endSection('payload');
        blank();
        blank();
        push('if __name__ == "__main__":');
        push('app.run(debug=False, host="127.0.0.1", port=os.getenv("PORT", "8000"))', 1);
        blank();
        return {
            ranges: ranges(),
            snippet: join()
        };
    }
};
