"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.rails = void 0;
var code_builder_1 = require("../../../../helpers/code-builder");
var escape_1 = require("../../../../helpers/escape");
exports.rails = {
    info: {
        key: 'rails',
        title: 'Rails',
        link: 'https://rubyonrails.org/',
        description: 'ReadMe Metrics Webhooks SDK usage on Rails'
    },
    convert: function (_a, options) {
        var secret = _a.secret, security = _a.security, server = _a.server;
        var opts = __assign({ indent: '  ' }, options);
        var _b = new code_builder_1.CodeBuilder({
            indent: opts.indent
        }), blank = _b.blank, endSection = _b.endSection, join = _b.join, push = _b.push, pushVariable = _b.pushVariable, ranges = _b.ranges, startSection = _b.startSection;
        push("require 'readme/webhook'");
        blank();
        push('class MetricsController < ApplicationController');
        push('def index', 1);
        push("render json: { message: 'hello world' }", 2);
        push('end', 1);
        blank();
        push('def post', 1);
        push('head :ok', 2);
        push('end', 1);
        blank();
        push('def webhook', 1);
        startSection('verification');
        push('# Your ReadMe secret', 2);
        push("secret = '".concat(secret, "'"), 2);
        push('# Verify the request is legitimate and came from ReadMe', 2);
        push("signature = request.headers['readme-signature']", 2);
        blank();
        push('begin', 2);
        push('Readme::Webhook.verify(request.raw_post, signature, secret)', 3);
        push('rescue Readme::MissingSignatureError, Readme::ExpiredSignatureError, Readme::InvalidSignatureError => e', 2);
        push('# Handle invalid requests', 3);
        push('render json: { error: e.message }, status: 401', 3);
        push('return', 3);
        push('end', 2);
        endSection('verification');
        blank();
        startSection('payload');
        push('# Fetch the user from the database and return their data for use with OpenAPI variables.', 2);
        push('# current_user ||= User.find(session[:user_id]) if session[:user_id]', 2);
        push('render json: {', 2);
        if (!server.length && !security.length) {
            push('# Add custom data to return in your webhook call here.', 3);
        }
        if (server.length) {
            push('# OAS Server variables', 3);
            server.forEach(function (data) {
                pushVariable("".concat((0, escape_1.escapeForObjectKey)(data.name), ": '").concat((0, escape_1.escapeForSingleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "',"), {
                    type: 'server',
                    name: data.name,
                    indentationLevel: 3
                });
            });
        }
        if (server.length && security.length) {
            blank();
        }
        if (security.length) {
            push('# OAS Security variables', 3);
            security.forEach(function (data) {
                if (data.type === 'http') {
                    // Only HTTP Basic auth has any special handling for supplying auth.
                    if (data.scheme === 'basic') {
                        pushVariable("".concat((0, escape_1.escapeForObjectKey)(data.name), ": { user: 'user', pass: 'pass' },"), {
                            type: 'security',
                            name: data.name,
                            indentationLevel: 3
                        });
                        return;
                    }
                }
                pushVariable("".concat((0, escape_1.escapeForObjectKey)(data.name), ": '").concat((0, escape_1.escapeForSingleQuotes)(data["default"] || data["default"] === '' ? data["default"] : data.name), "',"), {
                    type: 'security',
                    name: data.name,
                    indentationLevel: 3
                });
            });
        }
        push('}', 2);
        endSection('payload');
        push('end', 1);
        push('end');
        blank();
        return {
            ranges: ranges(),
            snippet: join()
        };
    }
};
